@@ -134,7 +134,7 @@ module Pluggaloid
     # [name] タグ名
     # ==== Return
     # Pluggaloid::HandlerTag
-    def handler_tag(slug=SecureRandom.uuid, name=slug)
+    def handler_tag(slug=SecureRandom.uuid, name=slug, &block)
       tag = case slug
             when String, Symbol
               vm.HandlerTag.new(slug: slug.to_sym, name: name.to_s, plugin: self)
@@ -143,9 +143,9 @@ module Pluggaloid
             else
               raise Pluggaloid::TypeError, "Argument `slug' must be instance of Symbol, String or Pluggaloid::HandlerTag, but given #{slug.class}."
             end
-      if block_given?
+      if block
         handlers = @events + @filters
-        yield tag
+        block.(tag)
         (@events + @filters - handlers).each do |handler|
           handler.add_tag(tag)
         end
@@ -157,9 +157,9 @@ module Pluggaloid
     # イベントリスナを列挙する
     # ==== Return
     # Set of Pluggaloid::Listener
-    def listeners
-      if block_given?
-        @events.each(&Proc.new)
+    def listeners(&block)
+      if block
+        @events.each(&block)
       else
         @events.dup
       end
@@ -168,9 +168,9 @@ module Pluggaloid
     # フィルタを列挙する
     # ==== Return
     # Set of Pluggaloid::Filter
-    def filters
-      if block_given?
-        @filters.each(&Proc.new)
+    def filters(&block)
+      if block
+        @filters.each(&block)
       else
         @filters.dup
       end
@@ -232,8 +232,7 @@ module Pluggaloid
       self end
 
     # プラグインが Plugin.uninstall される時に呼ばれるブロックを登録する。
-    def onunload
-      callback = Proc.new
+    def onunload(&callback)
       add_event(:unload) do |plugin_slug|
         if plugin_slug == self.name
           callback.call
@@ -246,15 +245,16 @@ module Pluggaloid
     # on_?name :: add_event(name)
     # filter_?name :: add_event_filter(name)
     def method_missing(method, *args, **kwrest, &proc)
-      case method.to_s
-      when /\Aon_?(.+)\Z/
-        add_event($1.to_sym, *args, **kwrest, &proc)
-      when /\Afilter_?(.+)\Z/
-        add_event_filter($1.to_sym, **kwrest, &proc)
-      when /\Ahook_?(.+)\Z/
-        add_event_hook($1.to_sym, &proc)
-      else
-        super end end
+      method_name = method.to_s
+      case
+      when method_name.start_with?('on')
+        event_name = method_name[(method_name[2] == '_' ? 3 : 2)..method_name.size]
+        add_event(event_name.to_sym, *args, **kwrest, &proc)
+      when method_name.start_with?('filter')
+        event_name = method_name[(method_name[6] == '_' ? 7 : 6)..method_name.size]
+        add_event_filter(event_name.to_sym, **kwrest, &proc)
+      end
+    end
 
     private
 
